<?php
/**
 * @author     Jay Leffue
 * @version    1.0
 */

/**
* This class is the tag of XML.
*/
class Tag
{
  /**
  * The name of this tag.
  */
  private $_name;

  /**
  * the array of properties to this tag.
  */
  private $_props;

  /**
  * Array of Tags that are direct chilren of this Tag.
  */
  private $_children;

  /**
  * The string content inside this tag.
  */
  private $_content;
  /**
  * class constructor
  *
  * @param    string    $TagName       the name of this tag
  * @param    array     $Attributes    an array of attributes this tag will have
  * @param    string    $Content       content that goes inside this tag.
  * @return   void
  */
  public function __construct($TagName, $Props = null, $Content = null)
  {
    $this->_name = $TagName;
    $this->_props = array();
    if(isset($Props))
    {
      $this->setProps($Props);
    }

    if(isset($Content))
    {
      $this->_content = $Content;
    }
    $this->_children = array();

  }


  /**
  * Returns the name used by this Tag.
  *
  * @return string
  */
  public function getName()
  {
    return $this->_name;
  }


  /**
  * Replaces the current properties for this tag and returns a reference to self.
  *
  * @param  array     $props An array of properties to set for this element.
  *
  * @throws Exception
  * @return Tag
  */
  public function setProps($Props)
  {
    if (!is_array($Props)) throw new Exception('Invalid Properties');

    $this->_props = $Props;
    return $this;
  }


  /**
  * Adds a specific property for this tag and returns a reference to self.
  * If the property already exists it will replace it.
  *
  * @param  mixed       $Props    An array of properties or if just setting one property a string indicating the name of the property to set.
  * @param  mixed       $Value    If $props is a string this will be the value to set for that property.
  *
  * @throws Exception
  * @return HTMLElement
  */
  public function addProps($Props,$Value=null)
  {
    if (is_array($Props))
    {
      $this->_props = array_merge($this->_props,$Props);
    }
    elseif(isset($Value))
    {
      $this->_props[$Props] = $Value;
    }
    else
    {
      unset($this->_props[$Props]);
    }
    return $this;
  }


  /**
  * Gets a specific property for this tag.
  *
  * @param     string    $Name    The name of the property to return.
  * @return    mixed
  */
  public function getProp($Name=null)
  {
    if (!isset($Name))
    {
      return $this->_props;
    }
    elseif(isset($this->_props[$Name]))
    {
      return $this->_props[$Name];
    }
    return null;
  }


  /**
  * Returns true if the specified property exists.
  *
  * @param    string    $Name    the name of the property we are searching for.
  *
  * @return boolean
  */
  public function hasProp($Name)
  {
    return array_key_exists($Name,$this->_props);
  }


  /**
  * Adds child tags to this tag and returns the tag it added.
  *
  * @param  Tag     $Child  The tag to make a child of this Tag.
  * @param  mixed   $Key    Optional value of the array key to assign to this array element
  *
  * @return Tag
  */
  public function addChild(Tag $Child,$Key=null)
  {
    if(isset($Key))
    {
      $this->_children[$Key] = $Child;
    }
    else
    {
      $this->_children[] = $Child;
    }
    return $Child;
  }


  /**
  * Looks to see if the tag has an immediate child with the specified name.
  *
  * @param  string     $Name  The Name we are looking for.
  *
  * @return boolean
  */
  public function hasChild($Name)
  {
    foreach($this->_children as $Child)
    {
      if($Child->getName() == $Name)
      {
        return true;
      }
    }

    return false;
  }


  /**
  * Returns the children.
  *
  * @param  none.
  *
  * @return array
  */
  public function getChildren()
  {
    return $this->_children;
  }


  /**
  * Returns the Child Tag with the name specified
  *
  * @param  $string    $Name    the name of the tag we hope to return
  *
  * @return Tag
  */
  public function getChild($Name)
  {
    foreach($this->_children as $Child)
    {
      if($Child->getName() == $Name)
      {
        return $Child;
      }
    }
  }


  public function getChildCount()
  {
    return count($this->_children);
  }


  /**
  * Returns the xml of this tag, if it has children tags, they will be called as well
  *
  * @param none
  *
  * @return string    The resulting XML
  */
  public function getTagXML()
  {
    $string = '';
    //if there are no children then just return the XML tag with embedded '/' at the end
    if(empty($this->_children))
    {
      if(isset($this->_content))
      {
        $string .= "<".$this->_name;
        foreach($this->_props as $name=>$prop)
        {
          $prop = ereg_replace("'",'',$prop); // Remove single quotes as these appear to be causing issues with fusion charts.
          $string .= " ".$name."='".$prop."'";
        }
        $string .=">".$this->_content."</".$this->_name.">";

      }
      else
      {
        $string .= "<".$this->_name;
        foreach($this->_props as $name=>$prop)
        {
          $prop = ereg_replace("'",'',$prop); // Remove single quotes as these appear to be causing issues with fusion charts.
          $string .= " ".$name."='".$prop."'";
        }
        $string .=" />";
      }
    }
    //otherwise we will need to dig deeper before closing our tag
    else
    {
      $string.= "<".$this->_name;
      foreach($this->_props as $name=>$prop)
      {
        $prop = ereg_replace("'",'',$prop); // Remove single quotes as these appear to be causing issues with fusion charts.
        $string .= " ".$name."='".$prop."'";
      }
      $string .=">";

      //now go through its children
      foreach($this->_children as $Child)
      {
        $string .= $Child->getTagXML();
      }
      $string .= "</".$this->_name.">";
    }

    return $string;
  }
}
?>
